/* Bluefish HTML Editor
 * menu.c - the home of the pulldowns
 *
 * Copyright (C) 1998-1999 Olivier Sessink, Chris Mazuc and Roland Steinbach
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include "default_include.h"

#include <locale.h>

#include <string.h>
#include <dirent.h>
#include <sys/types.h>

#include <stdio.h>
#include <stdlib.h>

#include "bluefish.h"
#include "callbacks.h"
#include "html.h"
#include "coloursel.h"  /* sel_colour_cb() */
#include "html2.h"
#include "javascript.h"
#include "document.h"			/* file_open etc. */
#include "project.h" 		/* project_open etc. */
#include "interface.h"			/* show_about_window_cb() */
#include "stringlist.h" 		/* edit_stringlist() */
#include "init.h" 				/* save_config_files() */
#include "snr2.h" 				/* the new search and replace */
#include "wml.h"
#include "images.h"
#include "wizards.h"
#include "ref_dialog.h"
#include "configure.h"
#include "undo.h"
#include "gtk_easy.h"
#include "highlight.h"
#include "toolbars.h"


/***************************************************************************/
void save_menu_shortcuts();
void load_menu_shortcuts();
static void edit_some_list_lcb(GtkWidget * widget, gpointer data);
gchar *menu_translate(const gchar * path, gpointer data);
static void insert_char_cb(GtkWidget * widget, gint data);
void get_main_menu(GtkWidget * window, GtkWidget ** menubar);
static void toggle_cust_menubar_lcb(GtkWidget * w, gpointer data);
static void hide_cust_menubar_cb(GtkWidget * w, gpointer data);
void show_cust_menubar_cb(GtkWidget * w, gpointer data);


/***************************************************************************/

/* I wish there was a better way to show he names of the different chars */
/* as in actually show them in parenthesis */
static GtkItemFactoryEntry menu_items[] = {
	{N_("/_File"), NULL, NULL, 0, "<Branch>"},
	{N_("/File/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/File/_New"), "<control>N", file_new_cb, 0, NULL},
	{N_("/File/_Open"), "<control>O", file_open_cb, 0, NULL},
	{N_("/File/Open from the web"), NULL, open_from_web_cb, 0, NULL},
	{N_("/File/Open recent"), NULL, NULL, 0, "<Branch>"},
	{N_("/File/Insert"), NULL, file_insert_cb, 0, NULL},
	{N_("/File/sep2"), NULL, NULL, 0, "<Separator>"},
	{N_("/File/_Save"), "<control>S", file_save_cb, 0, NULL},
	{N_("/File/Save _as"), NULL, file_save_as_cb, 0, NULL},
	{N_("/File/Move to"), NULL, file_move_to_cb, 0, NULL},
	{N_("/File/Save all"), NULL, file_save_all_cb, 0, NULL},
	{N_("/File/sep3"), NULL, NULL, 0, "<Separator>"},
	{N_("/File/Close"), "<control>W", file_close_cb, 0, NULL},
	{N_("/File/Close all"), NULL, file_close_all_cb, 0, NULL},
	{N_("/File/sep4"), NULL, NULL, 0, "<Separator>"},
	{N_("/File/Print"), NULL, file_print_cb, 0, NULL},
	{N_("/File/sep5"), NULL, NULL, 0, "<Separator>"},
	{N_("/File/Quit"), "<control>Q", cya_later, 0, NULL},
	{N_("/_Edit"), NULL, NULL, 0, "<Branch>"},
	{N_("/_Edit/Tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Edit/Cut"), "<control>x", cut_cb, 0, NULL},
	{N_("/Edit/Copy"), "<control>c", copy_cb, 0, NULL},
	{N_("/Edit/Paste"), "<control>v", paste_cb, 0, NULL},
	{N_("/Edit/sep2"), NULL, NULL, 0, "<Separator>"},
	{N_("/Edit/Select all"), "<control>a", sel_all_cb, 0, NULL},
	{N_("/Edit/Go to line"), NULL, go_to_line_win_cb, 0, NULL},
	{N_("/Edit/sep3"), NULL, NULL, 0, "<Separator>"},
	{N_("/Edit/Find"), "F6", search_cb, 0, NULL},
	{N_("/Edit/New find"), NULL, new_search_cb, 0, NULL},
	{N_("/Edit/Find again"), "<shift>F6", search_again_cb, 0, NULL},
	{N_("/Edit/Replace"), "F7", replace_cb, 0, NULL},
	{N_("/Edit/New replace"), NULL, new_replace_cb, 0, NULL},
	{N_("/Edit/Replace again"), "<shift>F7", replace_again_cb, 0, NULL},
	{N_("/Edit/sep3"), NULL, NULL, 0, "<Separator>"},
	{N_("/Edit/Undo"), "<control>z", undo_cb, 0, NULL},
	{N_("/Edit/Redo"), "<control>r", redo_cb, 0, NULL},
	{N_("/Edit/Undo all"), NULL, undo_all_cb, 0, NULL},
	{N_("/Edit/Redo all"), NULL, redo_all_cb, 0, NULL},
	{N_("/_View"), NULL, NULL, 0, "<Branch>"},
	{N_("/_View/Tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/View/View in _Netscape"), NULL, view_in_netscape_cb, 0, NULL},
	{N_("/View/_Drag 'n Drop Window"), NULL, dndwin_cb, 0, NULL},
#ifdef WITH_INTERNAL_PREVIEW
   {N_("/View/_Preview "), "F4" , preview_cb, 0, NULL},
#endif
	{N_("/View/sep1"), NULL, NULL, 0, "<Separator>"},
	{N_("/View/Next document"), "F1", notebook_next_document_cb, 0, NULL},
	{N_("/View/Previous document"), "F2", notebook_prev_document_cb, 0, NULL},
	{N_("/View/sep3"), NULL, NULL, 0, "<Separator>"},
	{N_("/View/Highlight syntax"), NULL, toggle_highlight_syntax_cb, 0, "<ToggleItem>"},
	{N_("/View/Refresh highlighting"), "F5", refresh_hi_cb, 0, NULL},
	{N_("/_Project"), NULL, NULL, 0, "<Branch>"},
	{N_("/Project/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Project/Open"), "<control>P", project_open_cb, 0, NULL},
	{N_("/Project/Close"), NULL, project_close_cb, 0, NULL},
	{N_("/Project/Save"), NULL, project_save_cb, 0, NULL},
	{N_("/Project/Save as"), NULL, project_save_as_cb, 0, NULL},
	{N_("/Project/sep1"), NULL, NULL, 0, "<Separator>"},
	{N_("/Project/Edit"), NULL, project_edit_cb, 0, NULL},
	{N_("/Project/sep2"), NULL, NULL, 0, "<Separator>"},
	{N_("/Project/Add current document"), NULL, project_add_document_cb, 0, NULL},
	{N_("/Project/Add all open documents"), NULL, project_add_document_cb, 1, NULL},
	{N_("/_Project/Files"), NULL, NULL, 0, "<Branch>"},
	{N_("/Project/Files/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/_Tags"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Headings"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Headings/Tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Headings/H1"), "<alt>F1", general_html_cb, 18, NULL},
	{N_("/Tags/Headings/H2"), "<alt>F2", general_html_cb, 19, NULL},
	{N_("/Tags/Headings/H3"), "<alt>F3", general_html_cb, 20, NULL},
	{N_("/Tags/Headings/H4"), "<alt>F4", general_html_cb, 21, NULL},
	{N_("/Tags/Headings/H5"), "<alt>F5", general_html_cb, 22, NULL},
	{N_("/Tags/Headings/H6"), "<alt>F6", general_html_cb, 23, NULL},
	{N_("/Tags/Special"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Accent"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Accent/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Special/Accent/Uppercase"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Accent/Uppercase/A-I"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Accent/Uppercase/A-I/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Special/Accent/Uppercase/A-I/A grave"), NULL, insert_char_cb, 0, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/A acute"), NULL, insert_char_cb, 1, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/A circumflex"), NULL, insert_char_cb, 2, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/A tilde"), NULL, insert_char_cb, 3, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/AE ligature"), NULL, insert_char_cb, 4, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/A diaeresis"), NULL, insert_char_cb, 5, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/A ring"), NULL, insert_char_cb, 6, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/C cedilla"), NULL, insert_char_cb, 7, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/E grave"), NULL, insert_char_cb, 8, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/E acute"), NULL, insert_char_cb, 9, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/E circumflex"), NULL, insert_char_cb, 10, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/E diaeresis"), NULL, insert_char_cb, 11, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/I grave"), NULL, insert_char_cb, 12, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/I acute"), NULL, insert_char_cb, 13, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/I circumflex"), NULL, insert_char_cb, 14, NULL},
	{N_("/Tags/Special/Accent/Uppercase/A-I/I diaeresis"), NULL, insert_char_cb, 15, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/N tilde"), NULL, insert_char_cb, 16, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/O grave"), NULL, insert_char_cb, 17, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/O acute"), NULL, insert_char_cb, 18, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/O circumflex"), NULL, insert_char_cb, 19, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/O tilde"), NULL, insert_char_cb, 20, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/O diaeresis"), NULL, insert_char_cb, 21, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/O slash"), NULL, insert_char_cb, 22, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/U grave"), NULL, insert_char_cb, 23, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/U acute"), NULL, insert_char_cb, 24, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/U circumflex"), NULL, insert_char_cb, 25, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/U diaeresis"), NULL,
	 insert_char_cb, 26, NULL},
	{N_("/Tags/Special/Accent/Uppercase/J-Z/Y acute"), NULL, insert_char_cb,
	 27, NULL},
	{N_("/Tags/Special/Accent/Lowercase"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Accent/Lowercase/A-I"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Accent/Lowercase/A-I/tearoff1"), NULL, NULL, 0,
	 "<Tearoff>"},
	{N_("/Tags/Special/Accent/Lowercase/A-I/a grave"), NULL, insert_char_cb,
	 28, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/a acute"), NULL, insert_char_cb,
	 29, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/a circumflex"), NULL,
	 insert_char_cb, 30, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/a tilde"), NULL, insert_char_cb,
	 31, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/a ring"), NULL, insert_char_cb,
	 32, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/ae ligature"), NULL,
	 insert_char_cb, 33, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/a diaeresis"), NULL,
	 insert_char_cb, 34, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/c cedilla"), NULL, insert_char_cb,
	 35, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/e grave"), NULL, insert_char_cb,
	 36, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/e acute"), NULL, insert_char_cb,
	 37, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/e circumflex"), NULL,
	 insert_char_cb, 38, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/e diaeresis"), NULL,
	 insert_char_cb, 39, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/i grave"), NULL, insert_char_cb,
	 40, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/i acute"), NULL, insert_char_cb,
	 41, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/i circumflex"), NULL,
	 insert_char_cb, 42, NULL},
	{N_("/Tags/Special/Accent/Lowercase/A-I/i diaeresis"), NULL,
	 insert_char_cb, 43, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/tearoff1"), NULL, NULL, 0,
	 "<Tearoff>"},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/n tilde"), NULL, insert_char_cb,
	 44, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/o grave"), NULL, insert_char_cb,
	 45, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/o acute"), NULL, insert_char_cb,
	 46, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/o circumflex"), NULL,
	 insert_char_cb, 47, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/o tilde"), NULL, insert_char_cb,
	 48, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/o diaeresis"), NULL,
	 insert_char_cb, 49, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/o slash"), NULL, insert_char_cb,
	 50, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/u grave"), NULL, insert_char_cb,
	 51, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/u acute"), NULL, insert_char_cb,
	 52, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/u circumflex"), NULL,
	 insert_char_cb, 53, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/u diaeresis"), NULL,
	 insert_char_cb, 54, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/y acute"), NULL, insert_char_cb,
	 55, NULL},
	{N_("/Tags/Special/Accent/Lowercase/J-Z/y diaeresis"), NULL,
	 insert_char_cb, 56, NULL},
	{N_("/Tags/Special/Accent/separator"), NULL, NULL, 0, "<Separator>"},
	{N_("/Tags/Special/Accent/Umlaut"), NULL, insert_char_cb, 57, NULL},
	{N_("/Tags/Special/Accent/Acute"), NULL, insert_char_cb, 58, NULL},
	{N_("/Tags/Special/Accent/Cedilla"), NULL, insert_char_cb, 59, NULL},
	{N_("/Tags/Special/Currency"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Currency/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Special/Currency/Cent sign"), NULL, insert_char_cb,
	 60, NULL},
	{N_("/Tags/Special/Currency/Pound sterling"), NULL, insert_char_cb,
	 61, NULL},
	{N_("/Tags/Special/Currency/Currency sign"), NULL, insert_char_cb,
	 62, NULL},
	{N_("/Tags/Special/Currency/Yen sign"), NULL, insert_char_cb, 63, NULL},
	{N_("/Tags/Special/Math-Science"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Math-Science/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Special/Math-Science/Logical not sign"), "<alt>1", insert_char_cb,
	 64, NULL},
	{N_("/Tags/Special/Math-Science/Multiplication sign"), "<alt>8",
	 insert_char_cb, 65, NULL},
	{N_("/Tags/Special/Math-Science/Division sign"), "<alt>slash", insert_char_cb,
	 66, NULL},
	{N_("/Tags/Special/Math-Science/Plus-minus sign"), NULL, insert_char_cb,
	 67, NULL},
	{N_("/Tags/Special/Math-Science/Less-than sign"), "<alt>comma", insert_char_cb,
	 68, NULL},
	{N_("/Tags/Special/Math-Science/Greater-than sign"), "<alt>period", insert_char_cb,
	 69, NULL},
	{N_("/Tags/Special/Math-Science/Superscript 1"), NULL, insert_char_cb,
	 70, NULL},
	{N_("/Tags/Special/Math-Science/Superscript 2"), NULL, insert_char_cb,
	 71, NULL},
	{N_("/Tags/Special/Math-Science/Superscript 3"), NULL, insert_char_cb,
	 72, NULL},
	{N_("/Tags/Special/Math-Science/One quarter"), NULL, insert_char_cb,
	 73, NULL},
	{N_("/Tags/Special/Math-Science/One half"), NULL, insert_char_cb,
	 74, NULL},
	{N_("/Tags/Special/Math-Science/Three quarters"), NULL, insert_char_cb,
	 75, NULL},
	{N_("/Tags/Special/Math-Science/Degree sign"), NULL, insert_char_cb,
	 76, NULL},
	{N_("/Tags/Special/Non Latin"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Non Latin/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Special/Non Latin/ETH (Icelandic)"), NULL, insert_char_cb,
	 77, NULL},
	{N_("/Tags/Special/Non Latin/THORN (Icelandic)"), NULL, insert_char_cb,
	 78, NULL},
	{N_("/Tags/Special/Non Latin/eth (Icelandic)"), NULL, insert_char_cb,
	 79, NULL},
	{N_("/Tags/Special/Non Latin/thorn"), NULL, insert_char_cb, 80, NULL},
	{N_("/Tags/Special/Non Latin/Sharp s"), NULL, insert_char_cb, 81, NULL},
	{N_("/Tags/Special/Non Latin/Micro sign"), NULL, insert_char_cb,
	 82, NULL},
	{N_("/Tags/Special/Other"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Special/Other/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Special/Other/Non-breaking space"), NULL, insert_char_cb,
	 83, NULL},
	{N_("/Tags/Special/Other/Section sign"), NULL, insert_char_cb,
	 84, NULL},
	{N_("/Tags/Special/Other/Copyright sign"), NULL, insert_char_cb,
	 85, NULL},
	{N_("/Tags/Special/Other/Left angle quotes"), NULL, insert_char_cb,
	 86, NULL},
	{N_("/Tags/Special/Other/Right angle quotes"), NULL, insert_char_cb,
	 87, NULL},
	{N_("/Tags/Special/Other/Registered trademark"), NULL, insert_char_cb,
	 88, NULL},
	{N_("/Tags/Special/Other/Inverted exclamation"), NULL, insert_char_cb,
	 89, NULL},
	{N_("/Tags/Special/Other/Inverted question mark"), NULL, insert_char_cb,
	 90, NULL},
	{N_("/Tags/Special/Other/Feminine ordinal"), NULL, insert_char_cb,
	 91, NULL},
	{N_("/Tags/Special/Other/Masculine ordinal"), NULL, insert_char_cb,
	 92, NULL},
	{N_("/Tags/Special/Other/Pilcrow (paragraph sign)"), NULL, insert_char_cb,
	 93, NULL},
	{N_("/Tags/Special/Other/Broken bar"), NULL, insert_char_cb, 94, NULL},
	{N_("/Tags/Special/Other/Soft hypen"), "<alt>minus", insert_char_cb, 95, NULL},
	{N_("/Tags/Special/Other/Spacing macron"), NULL, insert_char_cb,
	 96, NULL},
	{N_("/Tags/Special/Other/Middle dot"), NULL, insert_char_cb,
	 97, NULL},
	{N_("/Tags/Formatting"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Formatting/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Formatting/Bold"), "<alt>b", general_html_cb, 1, NULL},
	{N_("/Tags/Formatting/Italic"), "<alt>i", general_html_cb, 2, NULL},
	{N_("/Tags/Formatting/Underline"), "<alt>u", general_html_cb, 3, NULL},
	{N_("/Tags/Formatting/Strikeout"), "<alt>s", general_html_cb, 4, NULL},
	{N_("/Tags/Formatting/Strong"), NULL, general_html_cb, 16, NULL},
	{N_("/Tags/Formatting/Emphasis"), NULL, general_html_cb, 17, NULL},
	{N_("/Tags/Formatting/<separator>"), NULL, NULL, 0, "<Separator>"},
	{N_("/Tags/Formatting/Paragraph"), "<alt>P", general_html_cb, 5, NULL},
	{N_("/Tags/Formatting/Break"),  "<Control><Alt>r", general_html_cb, 6, NULL},
	{N_("/Tags/Formatting/Break clear all"), NULL, general_html_cb, 41, NULL},
	{N_("/Tags/Formatting/Non-Breaking Space"), "<alt>n", general_html_cb, 7, NULL},
	{N_("/Tags/Formatting/<separator>"), NULL, NULL, 0, "<Separator>"},
	{N_("/Tags/Formatting/Font Size +1"), "<control>equal", general_html_cb, 11, NULL},
	{N_("/Tags/Formatting/Font Size -1"), "<control>minus", general_html_cb, 12, NULL},
	{N_("/Tags/Formatting/<separator>"), NULL, NULL, 0, "<Separator>"},
	{N_("/Tags/Formatting/Preformatted Text"), "<Control><Alt>p", general_html_cb, 13, NULL},
	{N_("/Tags/Formatting/Subscript"), "<alt><shift>s", general_html_cb, 14, NULL},
	{N_("/Tags/Formatting/Superscript"), "<alt><control>s", general_html_cb, 15, NULL},
	{N_("/Tags/Formatting/<separator>"), NULL, NULL, 0, "<Separator>"},
	{N_("/Tags/Formatting/Center"), "<alt>c", general_html_cb, 8, NULL},
	{N_("/Tags/Formatting/Align right"), "<Alt>r", general_html_cb, 9, NULL},
	{N_("/Tags/Formatting/<separator>"), NULL, NULL, 0, "<Separator>"},
	{N_("/Tags/Formatting/Comment"), NULL, general_html_cb, 10, NULL},
	{N_("/Tags/Table"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/Table/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/Table/Table"), NULL, general_html_cb, 24, NULL},
	{N_("/Tags/Table/Tablerow"), NULL, general_html_cb, 25, NULL},
	{N_("/Tags/Table/Tableheader"), NULL, general_html_cb, 26, NULL},
	{N_("/Tags/Table/Tabledata"), NULL, general_html_cb, 27, NULL},
	{N_("/Tags/Table/Tablecaption"), NULL, general_html_cb, 28, NULL},
	{N_("/Tags/List"), NULL, NULL, 0, "<Branch>"},
	{N_("/Tags/List/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Tags/List/Unordered list"), NULL, general_html_cb, 33, NULL},
	{N_("/Tags/List/Ordered list"), NULL, general_html_cb, 34, NULL},
	{N_("/Tags/List/List item"), "<alt>l", general_html_cb, 35, NULL},
	{N_("/Tags/List/Definition list"), NULL, general_html_cb, 36, NULL},
	{N_("/Tags/List/Definition term"), "<alt>d", general_html_cb, 37, NULL},
	{N_("/Tags/List/Definition"), NULL, general_html_cb, 38, NULL},
	{N_("/Tags/List/menu"), NULL, general_html_cb, 39, NULL},
	{N_("/Tags/Replace iso 8859-1 chars"), NULL, iso8859_1_replace_cb, 39, NULL},
	{N_("/Tags/Replace ascii chars"), NULL, ascii_replace_cb, 39, NULL},
	{N_("/Tags/Insert Generator META-Tag"), NULL, general_html_cb, 47, NULL},
	{N_("/_Dialogs"), NULL, NULL, 0, "<Branch>"},
	{N_("/Dialogs/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Dialogs/General"), NULL, NULL, 0, "<Branch>"},
	{N_("/Dialogs/General/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Dialogs/General/Quickstart"), "<alt>q", quickstart, 0, NULL},
	{N_("/Dialogs/General/DTD"), NULL, dtd_cb, 0, NULL},
	{N_("/Dialogs/General/Head"), NULL, head_cb, 0, NULL},
	{N_("/Dialogs/General/Body"), NULL, body_cb, 0, NULL},
	{N_("/Dialogs/General/Anchor"), "<alt>a", quickanchor_cb, 0, NULL},
	{N_("/Dialogs/General/Email"), NULL, email_cb, 0, NULL},
	{N_("/Dialogs/General/Rule"), "<alt>h", quickrule_cb, 0, NULL},
	{N_("/Dialogs/General/Font"), "<alt>f", fontdialog_cb, 0, NULL},
	{N_("/Dialogs/General/Basefont"), NULL, basefont_cb, 0, NULL},
	{N_("/Dialogs/General/Quicklist"), NULL, quicklist_cb, 0, NULL},
	{N_("/Dialogs/General/Meta"), "<alt>m", meta_cb, 0, NULL},
	{N_("/Dialogs/General/Embed"), NULL, embed_cb, 0, NULL},
	{N_("/Dialogs/General/Select colour"), NULL, sel_colour_cb, 0, NULL},
	{N_("/Dialogs/General/Insert time"), NULL, insert_time_cb, 0, NULL},
	{N_("/Dialogs/General/Insert image"), "<control>i", image_insert_dialog_cb, 0, NULL},
	{N_("/Dialogs/General/Insert thumbnail"), "<control>t", image_thumbnail_dialog_cb, 0, NULL},
	{N_("/Dialogs/Table"), NULL, NULL, 0, "<Branch>"},
	{N_("/Dialogs/Table/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Dialogs/Table/Table"), NULL, tabledialog_cb, 0, NULL},
	{N_("/Dialogs/Table/Tablerow"), NULL, tablerowdialog_cb, 0, NULL},
	{N_("/Dialogs/Table/Tablehead_cber"), NULL, tableheaddialog_cb, 0, NULL},
	{N_("/Dialogs/Table/Tabledata"), NULL, tabledatadialog_cb, 0, NULL},
	{N_("/Dialogs/CSS"), NULL, NULL, 0, "<Branch>"},
	{N_("/Dialogs/CSS/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Dialogs/CSS/Create style"), NULL, insert_in_stylesheet, 0, NULL},
	{N_("/Dialogs/CSS/Span"), NULL, insert_span, 0, NULL},
	{N_("/Dialogs/CSS/Div"), NULL, insert_div, 0, NULL},
	{N_("/Dialogs/CSS/Add class"), NULL, insert_class, 0, NULL},
	{N_("/Dialogs/CSS/Style"), NULL, general_html_cb, 42, NULL},
	{N_("/Dialogs/CSS/Link to stylesheet"), NULL, general_html_cb, 46, NULL},
	{N_("/Dialogs/Frame"), NULL, NULL, 0, "<Branch>"},
	{N_("/Dialogs/Frame/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Dialogs/Frame/Frameset"), NULL, framesetdialog_cb, 0, NULL},
	{N_("/Dialogs/Frame/Frame"), NULL, framedialog_cb, 0, NULL},
	{N_("/Dialogs/Form"), NULL, NULL, 0, "<Branch>"},
	{N_("/Dialogs/Form/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Dialogs/Form/Form"), NULL, formdialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Submit"), NULL, submitdialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Reset"), NULL, resetdialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Text"), NULL, textdialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Hidden"), NULL, hiddendialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Textarea"), NULL, textareadialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Radio buttons"), NULL, radiodialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Check buttons"), NULL, checkdialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Select"), NULL, selectdialog_cb, 0, NULL},
	{N_("/Dialogs/Form/Option"), NULL, optiondialog_cb, 0, NULL},
	{N_("/Dialogs/Javascript"), NULL, NULL, 0, "<Branch>"},
	{N_("/Dialogs/Javascript/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Dialogs/Javascript/Mouseover Script"), NULL, mouseover_script_cb, 0, NULL},
	{N_("/Dialogs/_WML"), NULL, NULL, 0, "<Branch>"},
	{N_("/Dialogs/WML/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Dialogs/WML/Standard Document"), NULL, general_wml_cb, 6, NULL},
	{N_("/Dialogs/WML/Card"), NULL, carddialog_cb, 0, NULL},
	{N_("/Dialogs/WML/Postfield"), NULL, postfielddialog_cb, 0, NULL},
	{N_("/Dialogs/WML/Go"), NULL, godialog_cb, 0, NULL},
	{N_("/Dialogs/WML/Do"), NULL, dodialog_cb, 0, NULL},
	{N_("/Dialogs/WML/Anchor"), NULL, anchordialog_cb, 0, NULL},
	{N_("/Dialogs/WML/Access"), NULL, accessdialog_cb, 0, NULL},
	{N_("/Dialogs/WML/sep11"), NULL, NULL, 0, "<Separator>"},
	{N_("/Dialogs/WML/Paragraph"), NULL, general_wml_cb, 1, NULL},
	{N_("/Dialogs/WML/Line Break"), NULL, general_wml_cb, 2, NULL},
	{N_("/Dialogs/WML/Italic"), NULL, general_wml_cb, 3, NULL},
	{N_("/Dialogs/WML/Non-Breaking Space"), NULL, general_wml_cb, 4, NULL},
	{N_("/Dialogs/WML/Bold"), NULL, general_wml_cb, 5, NULL},
	{N_("/Dialogs/WML/sep12"), NULL, NULL, 0, "<Separator>"},
	{N_("/Dialogs/WML/Prev"), NULL, general_wml_cb, 7, NULL},
	{N_("/Dialogs/WML/Refresh"), NULL, general_wml_cb, 8, NULL},
	{N_("/Dialogs/WML/Noop"), NULL, general_wml_cb, 9, NULL},
	{N_("/Dialogs/WML/sep13"), NULL, NULL, 0, "<Separator>"},
	{N_("/Dialogs/WML/Set Variable"), NULL, vardialog_cb, 0, NULL},
	{N_("/E_xternal"), NULL, NULL, 0, "<Branch>"},
	{N_("/External/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/External/Weblint"), NULL, run_weblint_cb, 0, NULL},
	{N_("/External/sep1"), NULL, NULL, 0, "<Separator>"},
	{N_("/External/Filters"), NULL, NULL, 0, "<Branch>"},
	{N_("/External/Filters/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/External/Commands"), NULL, NULL, 0, "<Branch>"},
	{N_("/External/Commands/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/_Options"), NULL, NULL, 0, "<Branch>"},
	{N_("/Options/tearoff1"), NULL, NULL, 0, "<Tearoff>"},
	{N_("/Options/Preferences"), NULL, configure_cb, 0, NULL},
	{N_("/Options/View Main toolbar"), NULL, toggle_main_toolbar_lcb, 0, "<ToggleItem>"},
	{N_("/Options/View HTML toolbar"), NULL, toggle_html_toolbar_lcb, 0, "<ToggleItem>"},
	{N_("/Options/View Custom toolbar"), NULL, toggle_cust_menubar_lcb, 0, "<ToggleItem>"},
	{N_("/Options/sep2"), NULL, NULL, 0, "<Separator>"},
	{N_("/Options/_Lists"), NULL, NULL, 0, "<Branch>"},
	{N_("/Options/Lists/External filters"), NULL, edit_some_list_lcb, 1, NULL},
	{N_("/Options/Lists/External programs"), NULL, edit_some_list_lcb, 2, NULL},
	{N_("/Options/Lists/Syntax highlighting"), NULL, edit_some_list_lcb, 3, NULL},
	{N_("/Options/Lists/Reset syntax highlighting"), NULL, reset_syntax_highlighting_lcb, 0, NULL},
	{N_("/Options/sep1"), NULL, NULL, 0, "<Separator>"},
	{N_("/Options/Save config"), NULL, save_config_files, 0, NULL},
	{N_("/Options/Save shortcut key settings"), NULL, save_menu_shortcuts, 0, NULL},
	{N_("/_Help"), NULL, NULL, 0, "<Branch>"},
	{N_("/Help/_About"), NULL, show_about_window_cb, 0, NULL}
};

static gchar *sp_chars[] = {
	"&Agrave;", "&Aacute;", "&Acirc;", "&Atilde;", "&AElig;",
	"&Auml;", "&Aring;", "&Ccedil;", "&Egrave;", "&Eacute;",
	"&Ecirc;", "&Euml;", "&Igrave;", "&Iacute;", "&Icirc;",
	"&Iuml;", "&Ntilde;", "&Ograve;", "&Oacute;", "&Ocirc;",
	"&Otilde;", "&Ouml;", "&Oslash;", "&Ugrave;", "&Uacute;",
	"&Ucirc;", "&Uuml;", "&Yacute;", "&agrave;", "&aacute;",
	"&acirc;", "&atilde;", "&aring;", "&aelig;", "&auml;",
	"&ccedil;", "&egrave;", "&eacute;", "&ecirc;", "&euml;",
	"&igrave;", "&iacute;", "&icircumflex;", "&iuml;", "&ntilde;",
	"&ograve;", "&oacute;", "&ocirc;", "&otilde;", "&ouml;",
	"&oslash;", "&ugrave;", "&uacute;", "&ucirc;", "&uuml;",
	"&yacute;", "&yuml;", "&uml;", "&acute;", "&cedil;", "&cent;",
	"&pound;", "&curren;", "&yen;", "&not;", "&mult;", "&divide;",
	"&plusmn;", "&lt;", "&gt;", "&sup1;", "&sup2;", "&sup3;",
	"&frac14;", "&frac12;", "&frac34;", "&deg;", "&ETH;", "&THORN;",
	"&eth;", "&thorn;", "&szlig;", "&micro;", "&nbsp;", "&sect;",
	"&copy;", "&laquo;", "&raquo;", "&reg;", "&iexcl;", "&iquest;",
	"&ordf;", "&ordm;", "&para;", "&brvbar;", "&shy;", "&macr;",
	"&middot;"
};
#ifndef PATH_MAX
#define PATH_MAX 1024
#endif

void save_menu_shortcuts() {
	gchar *filename;
	
	filename = g_malloc(PATH_MAX);
	filename = strncat(strncpy(filename, g_get_home_dir(), PATH_MAX), "/.bluefish/menudump", PATH_MAX);
	gtk_item_factory_dump_rc(filename, NULL, TRUE);
	g_free(filename);
}

void load_menu_shortcuts() {
	gchar *filename;
	
	filename = g_malloc(PATH_MAX);
	filename = strncat(strncpy(filename, g_get_home_dir(), PATH_MAX), "/.bluefish/menudump", PATH_MAX);
	gtk_item_factory_parse_rc(filename);
	g_free(filename);
}


static void edit_some_list_lcb(GtkWidget * widget, gpointer data)
{
	switch (GPOINTER_TO_INT(data)) {
	case 1:
		{
		gchar *titles[] = {"name", "command", NULL};
		estrl_dialog(&main_v->props.external_filters, _("Bluefish configuration - external filters"), 7, 2, &titles[0]);
		}
		break;
	case 2:
		{
		gchar *titles[] = {"name", "command", NULL};
		estrl_dialog(&main_v->props.external_commands, _("Bluefish configuration - external commands"), 7, 2, &titles[0]);
		}
		break;
	case 3:
		{
		gchar *titles[] = {"color", "regex pattern", NULL};
		estrl_dialog(&main_v->props.syntax_configstrings, _("Bluefish configuration - syntax highlighting"), 8, 2, &titles[0]);
		}
		break;
	default:

		break;
	}
}


/***************************************************************************/
/* takes care of inserting the special characters */
static void insert_char_cb(GtkWidget * widget, gint data)
{
	insert_text(sp_chars[data], "");
}

/* 
 * menu factory crap, thanks to the gtk tutorial for this
 * both the 1.0 and the 1.2 code is directly from the tutorial
 */
void get_main_menu(GtkWidget * window, GtkWidget ** menubar)
{
	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;
	gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
	accel_group = gtk_accel_group_new();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_group);
#ifdef ENABLE_NLS
	gtk_item_factory_set_translate_func(item_factory, menu_translate, "<main>", NULL);
#endif
	gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
	gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
	if (menubar)
		*menubar = gtk_item_factory_get_widget(item_factory, "<main>");
	setup_toggle_item(item_factory, "/Options/View Main toolbar", main_v->props.v_main_tb);
	setup_toggle_item(item_factory, "/Options/View HTML toolbar", main_v->props.v_html_tb);
	setup_toggle_item(item_factory, "/Options/View Custom toolbar", main_v->props.v_custom_tb);
	setup_toggle_item(item_factory, "/View/Highlight syntax", main_v->props.cont_highlight_update);
}


/*********************************************************************/
#define MAX_TEXT_ENTRY 10

typedef struct {
	GtkWidget *dialog;
	GtkWidget *textentry[MAX_TEXT_ENTRY];
	gchar **array;
} Tcust_con_struc;

/*
Tcust_con_struc->array[0] = title
Tcust_con_struc->array[1] = formatstring before, containing %0, %1... that should be replaced by the 
				values from the dialog
Tcust_con_struc->array[2] = formatstring after
Tcust_con_struc->array[3] = number of variables from the dialog
Tcust_con_struc->array[4..] = the description of those variables
*/


static gchar *replace_with(gchar numc, gchar **array) {
	gint numi, i;

	numi = ((int) numc) -48;
	for (i=0; array[i] != NULL; i++) {
		DEBUG_MSG("replace_with, i=%d\n", i);
		if (i == numi) {
			DEBUG_MSG("replace_with, returning %d= %s\n", i, array[i]);
			return array[i];
		}
	}
	return NULL;
}


static gchar *replace_string(gchar *in_string, gchar **array) {
	gchar *tmp3;
	gchar *found, *search_in, *freestr;
	gchar *newstring, *tmpnewstring, *begin, *replace_w;

	newstring = g_strdup("");
	freestr = search_in = begin = g_strdup(in_string);
	while ( (search_in) && (found = strchr(search_in, '%')) ) {
		tmp3 = found;
		tmp3++;
		if ((replace_w = replace_with(*tmp3, array))) {
			*found = '\0';
			tmpnewstring = newstring;
			newstring = g_strconcat(tmpnewstring, begin, replace_w, NULL);
			DEBUG_MSG("replace_string, newstring=%s\n", newstring);
			g_free(tmpnewstring);
			begin = ++tmp3;
			search_in = tmp3;
		} else if (*tmp3 == '\0') {
			search_in = NULL;
		} else {
			search_in = found++;
		}
	}
	tmpnewstring = newstring;
	newstring = g_strconcat(tmpnewstring, begin,NULL);
	DEBUG_MSG("replace_string, newstring=%s\n", newstring);
	g_free(tmpnewstring);
	
	g_free(freestr);
	return newstring;
}

static void cust_con_struc_dialog_destroy_lcb(GtkWidget *widget, GdkEvent *event,  Tcust_con_struc *ccs) {
	window_destroy(ccs->dialog);
	g_free(ccs);
}

static void cust_con_struc_dialog_cancel_lcb(GtkWidget *widget, gpointer data) {
	cust_con_struc_dialog_destroy_lcb(NULL, NULL, data);
}

static void cust_con_struc_dialog_ok_lcb(GtkWidget *widget, Tcust_con_struc *ccs) {
	gchar **dialogarray;
	gint num_vars, i;
	gchar *before=NULL, *after=NULL;

	DEBUG_MSG("cust_con_struc_dialog_ok_lcb, ccs at %p\n", ccs);
	DEBUG_MSG("cust_con_struc_dialog_ok_lcb, array at %p, &array[0]=%p\n", ccs->array, &ccs->array[0]);
	DEBUG_MSG("cust_con_struc_dialog_ok_lcb, array[0] at %p, *array=%p\n", ccs->array[0], *ccs->array);
	num_vars = atoi(ccs->array[3]);
	DEBUG_MSG("cust_con_struc_dialog_ok_lcb, num_vars=%d, ccs->array[3]=%s\n", num_vars, ccs->array[3]);
	dialogarray = g_malloc((num_vars+1) * sizeof(char *));
	for (i=0; i<num_vars; i++) {
		dialogarray[i] = gtk_editable_get_chars(GTK_EDITABLE(ccs->textentry[i]), 0, -1);
		DEBUG_MSG("cust_con_struc_dialog_ok_lcb, i=%d, dialogarray[i]=%s\n", i, dialogarray[i]);
	}
	dialogarray[i] = NULL;
	DEBUG_MSG("cust_con_struc_dialog_ok_lcb, i=%d, dialogarray[i]=%p\n", i, dialogarray[i]);

	if (strlen(ccs->array[1])) {
		before = replace_string(ccs->array[1], dialogarray);
	}
	if (strlen(ccs->array[2])) {
		after = replace_string(ccs->array[2], dialogarray);
	}
	insert_text(before, after);
	g_strfreev(dialogarray);

	if (before) {
		g_free(before);
	}
	if (after) {
		g_free(after);
	}
	cust_con_struc_dialog_cancel_lcb(NULL, ccs);
}

static void cust_con_struc_dialog(gchar **array) {
	Tcust_con_struc *ccs;
	GtkWidget *vbox, *hbox, *okb, *cancb;
	gint i, num_vars;

	ccs = g_malloc(sizeof(Tcust_con_struc));
	DEBUG_MSG("cust_con_struc_dialog_cb, ccs at %p\n", ccs);
	ccs->array = array;
	DEBUG_MSG("cust_con_struc_dialog_cb, array at %p, &array[0]=%p\n", ccs->array, &ccs->array[0]);
	DEBUG_MSG("cust_con_struc_dialog_cb, array[0] at %p, *array=%p\n", ccs->array[0], *ccs->array);
	ccs->dialog = window_full(ccs->array[0], GTK_WIN_POS_MOUSE, GTK_WINDOW_DIALOG, 
			5, cust_con_struc_dialog_cancel_lcb, ccs);
	vbox = gtk_vbox_new(TRUE, 0);
	gtk_container_add(GTK_CONTAINER(ccs->dialog), vbox);
	DEBUG_MSG("cust_con_struc_dialog_cb, ccs->array[0]=%s\n", ccs->array[0]);
	
	num_vars = atoi(ccs->array[3]);
	DEBUG_MSG("cust_con_struc_dialog_cb, num_vars=%d, ccs->array[3]=%s\n", num_vars, ccs->array[3]);

	for (i=0; i<num_vars; i++) {
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(ccs->array[i+4]), TRUE, TRUE, 0);
		ccs->textentry[i] = gtk_entry_new();
		DEBUG_MSG("cust_con_struc_dialog_cb, textentry[%d]=%p\n", i, ccs->textentry[i]);
		gtk_box_pack_start(GTK_BOX(hbox), ccs->textentry[i], TRUE, TRUE, 0);		
		gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	}

	hbox = gtk_hbox_new(TRUE, 0);
	okb = bf_stock_ok_button(cust_con_struc_dialog_ok_lcb, ccs);
	cancb = bf_stock_cancel_button(cust_con_struc_dialog_cancel_lcb, ccs);
	gtk_box_pack_start(GTK_BOX(hbox), okb, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), cancb, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show_all(ccs->dialog);
}
/*****************************************************************************/


typedef struct {
	GtkItemFactoryEntry entry;
	gchar **array;
} Tcmenu_entry;

static void cust_menu_lcb(GtkWidget * widget, gpointer data) {
	Tcmenu_entry *cmentry;

	cmentry = (Tcmenu_entry *) g_list_nth_data(main_v->cmenu_arrays, GPOINTER_TO_INT(data));
	if (atoi(cmentry->array[3]) > 0) {
	     cust_con_struc_dialog(cmentry->array);
	} else {
	     insert_text (cmentry->array[1],cmentry->array[2]);
	}
}
#define TRY1

static void create_parent_and_tearoff(gchar *menupath, GtkItemFactory *ifactory) {
	char *basepath;
	GtkWidget *widg=NULL;
	GtkItemFactoryEntry entry;

	basepath = g_strndup(menupath, (strlen(menupath) - strlen(strrchr(menupath, '/'))));
	DEBUG_MSG("create_parent_and_tearoff, basepath=%s\n", basepath);
	widg = gtk_item_factory_get_widget(ifactory, basepath);
	if (!widg) {
		create_parent_and_tearoff(basepath, ifactory);
		entry.path = g_strconcat(basepath, "/sep", NULL);
		entry.accelerator = NULL;
		entry.callback = NULL;
		entry.callback_action = 0;
		entry.item_type = "<Tearoff>";
		gtk_item_factory_create_item(ifactory, &entry, NULL, 2);
		g_free(entry.path);
	}

}

static void fill_cust_menubar() {
	GtkItemFactory *ifactory;
	gint count;
	gchar **splittedstring;
	GList *tmplist;
	Tcmenu_entry *cmentry;

	ifactory = gtk_item_factory_from_widget(main_v->cmenu);

	tmplist = g_list_first(main_v->cmenu_arrays);
	while (tmplist) {
		cmentry = (Tcmenu_entry *) tmplist->data;
		gtk_item_factory_delete_entry(ifactory, &cmentry->entry);
		DEBUG_MSG("make_cust_menubar, removed entry.path=%s\n", cmentry->entry.path);
		g_free(cmentry->entry.path);
		g_free(cmentry);
		tmplist = g_list_next(tmplist);
	}
	g_list_free(main_v->cmenu_arrays);
	main_v->cmenu_arrays = NULL;

	count = 0;
	tmplist = g_list_first(main_v->props.cust_menu);
	while (tmplist) {
		splittedstring = (gchar **) tmplist->data;
		if (count_array(splittedstring) < 4) {
			DEBUG_MSG("make_cust_menubar, array count < 4\n");
			tmplist = g_list_next(tmplist);
			continue;
		}
		cmentry = g_malloc0(sizeof(Tcmenu_entry));
		cmentry->entry.path = g_strdup(splittedstring[0]);
		DEBUG_MSG("make_cust_menubar, entry.path=%s\n", cmentry->entry.path);
		cmentry->entry.callback = cust_menu_lcb;
		cmentry->entry.callback_action = count;
		cmentry->array = splittedstring;
		create_parent_and_tearoff(cmentry->entry.path, ifactory);
		gtk_item_factory_create_item(ifactory, &cmentry->entry, GINT_TO_POINTER(count), 2);
		main_v->cmenu_arrays = g_list_append(main_v->cmenu_arrays, cmentry);
		count++;
		tmplist = g_list_next(tmplist);
	}

}

void reset_cust_menu_list_cb(GtkWidget * widget, gpointer data)
{
	gchar *one[]= {"/php/ibase/ibase fetch row", "while ($%1 = ibase_fetch_row($%0)) {\n	echo $%1[0];\n", "\n}\nibase_free_result($%0);\n", "2", "ibase result variable name", "row variable name", NULL};
	gchar *two[]= {"/php/mysql/mysql fetch row", "while ($%1 = mysql_fetch_row($%0)) {\n	echo $%1[0];\n", "\n}\nmysql_free_result($%0);\n", "2", "mysql result variable name", "row variable name", NULL};
	gchar *three[]= {"/php/mysql/mysql connect", "$mysql_id = mysql_connect('%0', '%1', '%2');\nmysql_select_db($mysql_id, '%3');\n", "", "4", "mysql host", "mysql username", "mysql password", "database name", NULL};
	gchar *four[]= {"/php/ibase/ibase connect", "$ibase_id = ibase_connect('%0:%1', '%2', '%3');\n", "", "4", "ibase host", "ibase database path", "ibase username", "ibase password", NULL};
	gchar *five[]= {"/php/general/while each in array list", "while (list($key, $val) = each($%0)) {\n	", "\n}\n", "1", "array name", NULL};
	gchar *six[]= {"/sql/select all from table", "SELECT * FROM %0", "", "1", "table name", NULL};
	gchar *seven[] = {"/php/general/mail", "mail(\"%0\", \"%1\", $%2);\n", "", "3", "email adress", "subject", "mail body variable name",  NULL};
	gchar *eight[] = {"/php/general/for loop", "for ($%0 = %1; $%0 <= %2; $%0++) {\n ", "\n}\n", "3", "loop variable name", "from value", "to value",  NULL};
	gchar *nine[] = {"/php/ibase/query", "$%0 = ibase_query(%1, %2);\nif (ibase_errmsg() != 0){\n	echo 'Error: '.%2.' returned '.ibase_errmsg().\"<br>\\n\";\n}\n", "", "3", "result variable name", "ibase connection id name", "query string (quoted) or variable (including $)",  NULL};
	gchar *ten[] = {"/php/general/include file", "include('%0');\n", "", "1", "filename to include", NULL};
	gchar *eleven[] = {"/php/general/require file", "require('%0');\n", "", "1", "filename to require", NULL};
	gchar *twelve[] = {"/php/general/if isset", "if (isset($%0)) {\n	", "\n}", "1", "variable name to test for", NULL};
	gchar *thirteen[] = {"/php/general/if not isset", "if (!isset($%0)) {\n	", "\n}", "1", "variable name to test for", NULL};
	gchar *fourteen[] = {"/php/mysql/query", "$%0 = mysql_query(%1, %2);\n", "", "3", "result variable name", "mysql connection id name", "query string (quoted) or variable (including $)",  NULL};
	gchar *fifteen[] = {"/html/font symbol", "<font face=\"Symbol\">", "</font>", "0", NULL};
	gchar *sixteen[] = {"/html/div with class", "<div class=\"%0\">", "</div>", "1", "CSS class name", NULL};
	gchar *seventeen[] = {"/html/span with class", "<span class=\"%0\">", "</span>", "1", "CSS class name", NULL};
	gchar *eightteen[]= {"/php/php block", "\n<?php\n", "\n?>\n", "0", NULL};
	gchar *nineteen[]= {"/html/made with bluefish", "<a href=\"http://bluefish.openoffice.nl/\">Made with the Bluefish HTML editor</a>", "", "0", NULL};
	gchar *twenty[]= {"/html/meta keywords", "<meta name=\"keywords\" value=\"%0\">", "", "1", "keywords, space separated", NULL};
	gchar *twentyone[]= {"/html/meta refresh (client pull)", "<meta http-equiv=\"refresh\" content=\"0%; URL=%1\">", "", "2", "refresh time, in seconds", "URL to redirect to", NULL};
	free_arraylist(main_v->props.cust_menu);
	
	main_v->props.cust_menu = NULL;
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(one, sizeof(one)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(two, sizeof(two)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(three, sizeof(three)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(four, sizeof(four)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(five, sizeof(five)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(six, sizeof(six)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(seven, sizeof(seven)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(eight, sizeof(eight)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(nine, sizeof(nine)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(ten, sizeof(ten)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(eleven, sizeof(eleven)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(twelve, sizeof(twelve)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(thirteen, sizeof(thirteen)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(fourteen, sizeof(fourteen)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(fifteen, sizeof(fifteen)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(sixteen, sizeof(sixteen)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(seventeen, sizeof(seventeen)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(eightteen, sizeof(eightteen)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(nineteen, sizeof(nineteen)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(twenty, sizeof(twenty)));
	main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, g_memdup(twentyone, sizeof(twentyone)));
	fill_cust_menubar();
}

/*******************************************************************/
/*               Custom menu editor                                */
/*******************************************************************/

typedef struct {
	GtkWidget *win;
	GtkWidget *clist;
	GtkWidget *menupath;
	GtkWidget *before;
	GtkWidget *after;
	GtkWidget *num;
	gchar **lastarray;
	GtkWidget *dynvbox;
	GtkWidget *hboxes[MAX_TEXT_ENTRY];
	GtkWidget *descriptions[MAX_TEXT_ENTRY];
	GList *worklist;
} Tcmenu_editor;

static void cme_destroy_lcb(GtkWidget *widget, GdkEvent *event, Tcmenu_editor* cme) {
	window_destroy(cme->win);
	free_arraylist(cme->worklist);
	g_free(cme);
}

static void cme_close_lcb(GtkWidget *widget, gpointer data) {
	cme_destroy_lcb(NULL, NULL, data);
}

static void cme_ok_lcb(GtkWidget *widget, Tcmenu_editor *cme) {
	GList *tmplist;

	tmplist = main_v->props.cust_menu;
	main_v->props.cust_menu = cme->worklist;
	cme->worklist = tmplist;
	cme_destroy_lcb(NULL, NULL, cme);
	fill_cust_menubar();
}

static void cme_clist_unselect_lcb(GtkWidget * widget, gint row, gint column, GdkEventButton * event, Tcmenu_editor *cme) {
	gint i;

	gtk_entry_set_text(GTK_ENTRY(cme->menupath), "");
	gtk_editable_delete_text(GTK_EDITABLE(cme->before), 0, -1);
	gtk_editable_delete_text(GTK_EDITABLE(cme->after), 0, -1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(cme->num), 0);
	for (i = 0 ; i < MAX_TEXT_ENTRY; i++) {
		gtk_entry_set_text(GTK_ENTRY(cme->descriptions[i]), "");
	}
	cme->lastarray = NULL;
	DEBUG_MSG("cme_clist_unselect_lcb, lastarray=%p\n", cme->lastarray);
}

static void cme_create_entries(Tcmenu_editor *cme, gint num) {
	gint i;

	for (i = 0; i < MAX_TEXT_ENTRY ; i++) {
		if (i < num) {
			gtk_widget_show(cme->hboxes[i]);
		} else {
			gtk_widget_hide(cme->hboxes[i]); 
		}
	}
}

static void cme_spin_changed_lcb(GtkWidget *widget, Tcmenu_editor *cme) {
	cme_create_entries(cme, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(cme->num)));
}

static void cme_clist_select_lcb(GtkWidget * widget, gint row, gint column, GdkEventButton * event, Tcmenu_editor *cme) {
	gint num=0, pos=0, i;

	DEBUG_MSG("cme_clist_select_lcb, row=%d\n", row);
	cme->lastarray = g_list_nth_data(cme->worklist, row);
	DEBUG_MSG("cme_clist_select_lcb, lastarray=%p\n", cme->lastarray);
	DEBUG_MSG("cme_clist_select_lcb, cme->lastarray[0]=%s\n", cme->lastarray[0]);
	gtk_entry_set_text(GTK_ENTRY(cme->menupath), cme->lastarray[0]);
	
	gtk_editable_delete_text(GTK_EDITABLE(cme->before), 0, -1);
	DEBUG_MSG("cme_clist_select_lcb, cme->lastarray[1]=%s\n", cme->lastarray[1]);
	gtk_editable_insert_text(GTK_EDITABLE(cme->before), cme->lastarray[1], strlen(cme->lastarray[1]), &pos);
	gtk_editable_delete_text(GTK_EDITABLE(cme->after), 0, -1);
	DEBUG_MSG("cme_clist_select_lcb, cme->lastarray[2]=%s\n", cme->lastarray[2]);
	pos = 0;
	gtk_editable_insert_text(GTK_EDITABLE(cme->after), cme->lastarray[2], strlen(cme->lastarray[2]), &pos);
	
	num = atoi(cme->lastarray[3]);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(cme->num), num);
	
	cme_create_entries(cme, num);
	for (i = 0 ; i < num; i++) {
		gtk_entry_set_text(GTK_ENTRY(cme->descriptions[i]), cme->lastarray[i+4]);
	}
	for (i = num ; i < MAX_TEXT_ENTRY; i++) {
		gtk_entry_set_text(GTK_ENTRY(cme->descriptions[i]), "");
	}
}

static gchar **cme_create_array(Tcmenu_editor *cme) {
	gchar **newarray;
	gint num, i;
	
	gtk_spin_button_update(GTK_SPIN_BUTTON(cme->num));
	num  = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(cme->num));
	DEBUG_MSG("cme_create_array, num=%d\n", num);
	newarray = g_malloc0((num+6) * sizeof(char *));
	newarray[0] = gtk_editable_get_chars(GTK_EDITABLE(cme->menupath), 0, -1);
	if (newarray[0][0] != '/') {
		DEBUG_MSG("cme_create_array, menupath does not start with slash, returning NULL\n");
		g_free(newarray[0]);
		g_free(newarray);
		return (NULL);
	}
	newarray[1] = gtk_editable_get_chars(GTK_EDITABLE(cme->before), 0, -1);
	newarray[2] = gtk_editable_get_chars(GTK_EDITABLE(cme->after), 0, -1);
	newarray[3] = gtk_editable_get_chars(GTK_EDITABLE(cme->num), 0, -1);

	DEBUG_MSG("cme_create_array, newarray[3]=%s\n", newarray[3]);
	for (i = 0 ; i < num; i++) {
		DEBUG_MSG("cme_create_array, adding descriptions[%d] to newarray[%d]\n", i, i+4);
		newarray[4+i] = gtk_editable_get_chars(GTK_EDITABLE(cme->descriptions[i]), 0, -1);
	}
	DEBUG_MSG("cme_create_array, setting newarray[%d] to NULL\n",i+4);
	newarray[4+i] = NULL;
	return newarray;
}

static void cme_add_lcb(GtkWidget *widget, Tcmenu_editor *cme) {
	gchar **newarray;
	gchar *tmplistentry[1]; 
	gint row;

	newarray = cme_create_array(cme);
	if (newarray != NULL){
	     cme->worklist = g_list_append(cme->worklist, newarray);
	     tmplistentry[0] = newarray[0];
	     row = gtk_clist_append(GTK_CLIST(cme->clist), tmplistentry);
		  cme->lastarray = newarray;
		  DEBUG_MSG("cme_add_lcb, about to select row %d\n", row);
		  gtk_clist_select_row(GTK_CLIST(cme->clist), row, 0);
	}
}

static void cme_update_lcb(GtkWidget *widget, Tcmenu_editor *cme) {
	GList *tmplist;
	gchar **newarray;
	gint row;

	if (cme->lastarray == NULL) {
		cme_add_lcb(NULL, cme);
		return;
	}

	newarray = cme_create_array(cme);
	if (newarray) {

		row = g_list_index(cme->worklist, cme->lastarray);
		tmplist = g_list_nth(cme->worklist, row);
		g_strfreev(cme->lastarray);
		DEBUG_MSG ("cme_update_lcb, row=%d\n", row);

		tmplist->data = newarray;
		gtk_clist_set_text(GTK_CLIST(cme->clist), row, 0, newarray[0]);

		cme->lastarray = g_list_nth_data(cme->worklist, row);
	} else {
		DEBUG_MSG ("cme_update_lcb, no new array, cancelled\n");
	}
	DEBUG_MSG ("cme_update_lcb finished\n");
}

static void cme_delete_lcb(GtkWidget *widget, Tcmenu_editor *cme) {
	gint row;
	gchar **array;
	
	if (cme->lastarray) {
		array = cme->lastarray;
		DEBUG_MSG("cme_delete_lcb, delete lastarray %p\n", array);
		row = g_list_index(cme->worklist, array);
		DEBUG_MSG("cme_delete_lcb, g_list_length=%d\n", g_list_length(cme->worklist));
		cme->worklist = g_list_remove(cme->worklist, array);
		DEBUG_MSG("cme_delete_lcb, row=%d, g_list_length=%d\n", row, g_list_length(cme->worklist));
		gtk_clist_remove(GTK_CLIST(cme->clist), row);
		g_strfreev(array);
		cme->lastarray = NULL;
	} else {
		DEBUG_MSG("cme_delete_lcb, lastarray=NULL, nothing to delete\n");
	}
}

void cmenu_editor(GtkWidget *widget, gpointer data) {
	Tcmenu_editor *cme;
	GtkWidget *hbox, *vbox,*frame, *vbox2, *but, *scrolwin;
	GList *tmplist;
	gchar *tmplistentry[1];
	gchar **splittedstring;
	gint i;
	gchar *tmpstr;
	
	cme = g_malloc0(sizeof(Tcmenu_editor));
	cme->win = window_full(_("Bluefish config - custom menu"), GTK_WIN_POS_NONE
			, GTK_WINDOW_DIALOG, 5, cme_destroy_lcb, cme);

	vbox = gtk_vbox_new(FALSE, 2);
	gtk_container_add(GTK_CONTAINER(cme->win), vbox);
	/* upper area */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	/* clist & menupath area */
	vbox2 = gtk_vbox_new(FALSE, 0);	
	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);

	gtk_box_pack_start(GTK_BOX(vbox2), gtk_label_new(_("Menu path")), FALSE, FALSE, 0);
	cme->menupath = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(vbox2),cme->menupath , FALSE, FALSE, 0);

	scrolwin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	cme->clist = gtk_clist_new(1);
	gtk_signal_connect(GTK_OBJECT(cme->clist), "select_row", GTK_SIGNAL_FUNC(cme_clist_select_lcb), cme);
	gtk_signal_connect(GTK_OBJECT(cme->clist), "unselect_row", GTK_SIGNAL_FUNC(cme_clist_unselect_lcb), cme);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolwin), cme->clist);
	gtk_widget_set_usize(scrolwin, 180, 250);
	gtk_box_pack_start(GTK_BOX(vbox2), scrolwin, TRUE, TRUE, 0);

	/* dynamic entries area */
	vbox2 = gtk_vbox_new(FALSE, 0);	
	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);

	gtk_box_pack_start(GTK_BOX(vbox2), gtk_label_new(_("Number of variables")), FALSE, FALSE, 0);		
	cme->num = spinbut_with_value(NULL, 0, MAX_TEXT_ENTRY, 1,1);
	gtk_signal_connect(GTK_OBJECT(cme->num), "changed", cme_spin_changed_lcb, cme);
	gtk_box_pack_start(GTK_BOX(vbox2),cme->num , FALSE, FALSE, 0);
	
	frame = gtk_frame_new(_("Variables"));
	gtk_box_pack_end(GTK_BOX(vbox2), frame, TRUE, TRUE, 0);
	cme->dynvbox = gtk_vbox_new(FALSE, 0);	
	gtk_container_add(GTK_CONTAINER(frame), cme->dynvbox);
	for (i = 0; i <  MAX_TEXT_ENTRY; i++) {
		cme->hboxes[i] = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(cme->dynvbox), cme->hboxes[i], FALSE, FALSE, 0);
		tmpstr = g_strdup_printf("%%%d", i);
		gtk_box_pack_start(GTK_BOX(cme->hboxes[i]), gtk_label_new(tmpstr), FALSE, FALSE, 0);
		g_free(tmpstr);
		cme->descriptions[i] = gtk_entry_new();
		gtk_box_pack_start(GTK_BOX(cme->hboxes[i]), cme->descriptions[i], TRUE, TRUE, 0);
	}

	/* lower area */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	/* before and after text area */
	vbox2 = gtk_vbox_new(FALSE, 0);	
	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), gtk_label_new(_("Formatstring before")), FALSE, FALSE, 0);
	cme->before = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(cme->before), TRUE);
	gtk_widget_set_usize(cme->before, 280, 50);
	gtk_box_pack_start(GTK_BOX(vbox2), cme->before, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), gtk_label_new(_("Formatstring after")), FALSE, FALSE, 0);
	cme->after = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(cme->after), TRUE);
	gtk_widget_set_usize(cme->after, 280, 50);
	gtk_box_pack_start(GTK_BOX(vbox2), cme->after, TRUE, TRUE, 0);
	
	/* button area */
	vbox2 = gtk_vbox_new(FALSE, 0);	
	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
	but = bf_stock_button(_("Add"), cme_add_lcb, cme);
	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 5);
	but = bf_stock_button(_("Update"), cme_update_lcb, cme);
	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 5);
	but = bf_stock_button(_("Delete"), cme_delete_lcb, cme);
	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 5);
	
	/* frame for cancel/ok buttons */
	frame = gtk_frame_new(NULL);
	gtk_box_pack_end(GTK_BOX(vbox2), frame, FALSE, FALSE, 0);
	hbox = gtk_hbox_new(TRUE, 5);
	gtk_container_add(GTK_CONTAINER(frame), hbox);
	but = bf_stock_ok_button(cme_ok_lcb, cme);
	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
	
	but = bf_stock_cancel_button(cme_close_lcb, cme);
	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
	/* ready !! */
	cme->worklist = duplicate_arraylist(main_v->props.cust_menu);

	tmplist = g_list_first(cme->worklist);
	while (tmplist) {
		splittedstring = (gchar **) tmplist->data;
		tmplistentry[0] = splittedstring[0];
		gtk_clist_append(GTK_CLIST(cme->clist), tmplistentry);
		tmplist = g_list_next(tmplist);
	}

	gtk_widget_show_all(cme->win);
}

/*************************************************************************/

void make_cust_menubar()
{
	static GtkItemFactoryEntry cust_menu[] = {
		{N_("/_Custom menu"), NULL, NULL, 0, "<Branch>"},
		{N_("/Custom menu/sep"), NULL, NULL, 0, "<Tearoff>"},
		{N_("/Custom menu/Edit custom menu"), NULL, cmenu_editor, 0, NULL},
		{N_("/Custom menu/Reset custom menu"), NULL, reset_cust_menu_list_cb, 0, NULL}
	};
	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;
	gint nmenu_items = sizeof(cust_menu) / sizeof(cust_menu[0]);

	DEBUG_MSG("make_cust_menubar, started\n");

	/* this should only happen once !!!!!!!!!! */
	accel_group = gtk_accel_group_new();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_group);
#ifdef ENABLE_NLS
	gtk_item_factory_set_translate_func(item_factory, menu_translate, "<main>", NULL);
#endif
	gtk_item_factory_create_items(item_factory, nmenu_items, cust_menu, NULL);
	gtk_window_add_accel_group(GTK_WINDOW(main_v->main_window), accel_group);

	main_v->cmenu = gtk_item_factory_get_widget(item_factory, "<main>");
	gtk_container_add(GTK_CONTAINER(cust_handle_box), main_v->cmenu);
	gtk_widget_show(main_v->cmenu);

	fill_cust_menubar();

	DEBUG_MSG("make_cust_menubar, finished\n");
}
static void toggle_cust_menubar_lcb(GtkWidget * w, gpointer data)
{
	if (GTK_WIDGET_VISIBLE(cust_handle_box)) {
		hide_cust_menubar_cb(NULL, NULL);
	} else {
		show_cust_menubar_cb(NULL, NULL);
	}
}

static void hide_cust_menubar_cb(GtkWidget * w, gpointer data)
{
	main_v->props.v_custom_tb = 0;
	gtk_widget_hide(cust_handle_box);
}

void show_cust_menubar_cb(GtkWidget * w, gpointer data)
{
	main_v->props.v_custom_tb = 1;
	if (!main_v->cmenu) {
		make_cust_menubar();
	}
	gtk_widget_show(cust_handle_box);
}



/*******************************************************************/
/*               Open Recent menu handling                         */
/*******************************************************************/

/* the only required header */
GtkWidget *create_recent_entry(char *filename, int append);
/*******************************************************************/

/* save_recent_list
 * Saves the items of main_v->recent_files to the .bluefish/recentlist file
 * so that they can be restored in the next Bluefish session */
void save_recent_list() {
	gchar *recentfile;
	GList *putlist=NULL, *list;

	DEBUG_MSG("save_recent_list, started\n");
	recentfile = g_strconcat(getenv("HOME"), "/.bluefish/recentlist", NULL);

	list = g_list_first(main_v->recent_files);	
	while (list) {
		putlist = add_to_stringlist(putlist, g_strdup(GTK_LABEL(GTK_BIN(list->data)->child)->label));
		list = g_list_next(list);
	}	
	
	put_stringlist(recentfile, putlist);
	free_stringlist(putlist);
	g_free(recentfile);
}

/* This function was nessecary I'm afraid, if you find a way
 * that it is not needed for open_recent_file_cb, please
 * send the patch. This also removes the last entry if
 * filename == 'last' */
GtkWidget *remove_recent_entry(gchar *filename) {
	GList *list;
	gpointer tmp;

	if(strcmp(filename, "last") ==0) {
		list = g_list_last(main_v->recent_files);
		tmp = list->data;
		main_v->recent_files = g_list_remove(main_v->recent_files, list->data);
		return tmp;
	}	else {
		list = g_list_first(main_v->recent_files);
		while (list) {
			DEBUG_MSG("remove_recent_entry, list=%p, data=%p\n", list, list->data);
			DEBUG_MSG("remove_recent_entry, list->data=%s\n",GTK_LABEL(GTK_BIN(list->data)->child)->label);
			if(!strcmp(GTK_LABEL(GTK_BIN(list->data)->child)->label, filename)) {
				tmp = list->data;
				main_v->recent_files = g_list_remove(main_v->recent_files, list->data);
				DEBUG_MSG("remove_recent_entry, returning %p\n", tmp);
				return tmp;
			}		
			list = g_list_next(list);
		}
	}

	return NULL;
}



/* open_recent_file
 * This function should be called when a menu from the Open Recent list
 * has been selected. Data contains its position in the main_v->recent_files */

static void open_recent_file_cb(gchar *data) {
	GtkWidget *tmp;
	gchar *filename;

	DEBUG_MSG("open_recent_file_cb, started\n");
	/* Get the filename, which begins from character 17 and on*/
	/* the syntax seems weird, but it works, don't you just love C? :) */
	filename = data;
	DEBUG_MSG("open_recent_file_cb, filename is %s\n", filename);

	/* Now, let's check if that file still exists and is readable, before loading it */
	if (!file_exists_and_readable(filename)) {
		return;
	}

	/* Open the file that was asked */
	doc_new_with_file(filename);
	DEBUG_MSG("open_recent_file_cb, file opened\n");

	/* Bring the chosen item first in the list */
	tmp = remove_recent_entry(filename);
	gtk_widget_hide(tmp);

	main_v->recent_files = g_list_prepend(main_v->recent_files,
								create_recent_entry(GTK_LABEL(GTK_BIN(tmp)->child)->label, 0));
	gtk_widget_destroy(tmp);

	/* save the new list */
	save_recent_list();

	return;
}

/* create_recent_entry
 * This function builds the gtkitemfactoryentry and inserts it at the
 * main_v->menubar. Furthermore, it returns a pointer to it, so that
 * this pointer can be added in the main_v->recent_files list */
GtkWidget *create_recent_entry(char *filename, int append) {
	GtkWidget *tmp, *menu;
	GtkItemFactory *factory;

	/* add it to main_v->menubar */
	factory = gtk_item_factory_from_widget(main_v->menubar);
	menu = gtk_item_factory_get_widget(factory, N_("<main>/File/Open recent"));

	tmp = gtk_menu_item_new_with_label(filename);
	gtk_signal_connect_object(GTK_OBJECT(tmp), "activate",
					GTK_SIGNAL_FUNC(open_recent_file_cb), (gpointer) GTK_LABEL(GTK_BIN(tmp)->child)->label);
/*	gtk_signal_connect_object(GTK_OBJECT(tmp), "activate",
					GTK_SIGNAL_FUNC(open_recent_file_cb), (gpointer) g_strdup("Hello World")); */

	gtk_widget_show(tmp);

	if(append == 1)
		gtk_menu_append(GTK_MENU(menu), tmp);
	else
		gtk_menu_prepend(GTK_MENU(menu), tmp);

	flush_queue();

	/* return it */
	return tmp;
}


/* open_recent_list()
 * Gets the list of documents from .bluefish/recentlist and inserts
 * it at the File-->Open Recent menu. If the file doesn't exist (probably
 * because this is the first time Bluefish is running) then a menu
 * item telling that no recent files exist will appear */

void open_recent_list() {
	gchar *recentfile;
	GList *filenames=NULL, *tmp;
	

	DEBUG_MSG("open_recent_list, started\n");
	recentfile = g_strconcat(getenv("HOME"), "/.bluefish/recentlist", NULL);
	filenames = get_stringlist(recentfile, filenames);
	if (!filenames) {
		filenames = add_to_stringlist(main_v->recent_files, "(none)");
	}
	
	tmp = g_list_first(filenames);
	while (tmp) {
		main_v->recent_files  = g_list_append(main_v->recent_files, create_recent_entry(tmp->data, 1));
		tmp = g_list_next(tmp);
	}
	
	free_stringlist(filenames);
	g_free(recentfile);
}

/* Add_to_recent_list
 * This should be called when a new file is opened, i.e. from
 * file_open_cb, it adds a new entry which also appears in the
 * menu bar, and (if nessecary) deletes the last entry */
void add_to_recent_list(gchar *filename) {
	GtkWidget *tmp;

	DEBUG_MSG("add_to_recent_list, started\n");
	/* First of all, create the entry and insert it at the list*/
	main_v->recent_files = g_list_prepend(main_v->recent_files,
								create_recent_entry(filename, 0));

	DEBUG_MSG("add_to_recent_list, inserted item\n");

	if(g_list_length(main_v->recent_files) > main_v->props.max_recent_files) {
		tmp = remove_recent_entry("last");
		DEBUG_MSG("add_to_recent_list, entry %s to be deleted\n", GTK_LABEL(GTK_BIN(tmp)->child)->label);
		gtk_widget_hide(tmp);
		gtk_widget_destroy(tmp);
	}

	flush_queue();

	DEBUG_MSG("add_to_recent_list, trying to save\n");
	/* save the new list */
	save_recent_list();

	return;
}

/*******************************************************************/
/*           End of Open Recent menu handling                      */
/*******************************************************************/


#ifdef ENABLE_NLS

gchar *menu_translate(const gchar * path, gpointer data)
{
	static gchar *menupath = NULL;

	gchar *retval;
	gchar *factory;

	factory = (gchar *) data;

	if (menupath)
		g_free(menupath);

	menupath = g_strdup(path);

	if ((strstr(path, "/tearoff1") != NULL) || (strstr(path, "/---") != NULL) || (strstr(path, "/MRU") != NULL))
		return menupath;

	retval = gettext(menupath);

	return retval;
}

#endif							/*  ENABLE_NLS  */
